%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                       % 
% Gegeben die folgenden Operatordefinitionen:           %
% :- op(300, xfx, [are, is_a]).                         %
% :- op(300, fx, likes).                                %
% :- op(200, xfy, and).                                 %
% :- op(100, fy, famous).                               %
% :- op(500, xf, or_not).                               %
%                                                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Welche der folgenden Ausdrücke sind wohlgeformt? Wie klammert Prolog die Ausdrücke intern?
?- write_canonical(X is_a witch or_not).
?- write_canonical(harry and ron and hermione are friends).
?- write_canonical(harry is_a wizard and likes quidditch).
?- write_canonical(dumbledore is_a famous famous wizard or_not).
?- write_canonical(famous harry and ron are wizards).
?- write_canonical(ron is_a wizards and harry likes quidditch).

% Erklärung

% op\3
% op(+Precedence, +Type, :Name)
% Precedence: Beschreibt die Priorität bei der Evaluation des Operators. 
% Liegt zwischen 0 und 1200. Ein Wert von 0 deaktivert den Operator.
% Type: Beschreibt die Position des Funktors (f) in Relation zu den Argumenten (x,y).
% Es gilt, dass die "Precedence" von x echt kleiner als die von f ist und die von y kleiner gleich der von f.
% Mögliche Werte: xf, yf, xfx, xfy, yfx, fy oder fx
% Name: Name des Operator, entweder ein String oder eine Liste von Strings.
% Für weitere Informationen siehe: https://www.swi-prolog.org/pldoc/man?predicate=op/3



%%% Lösung:

% 1. ?- write_canonical(X is_a witch or_not).
% --> or_not(is_a(_,witch)), true

% 2. ?- write_canonical(harry and ron and hermione are friends).
% --> are(and(harry,and(ron,hermione)),friends), true

% 3. ?- write_canonical(harry is_a wizard and likes quidditch).
% --> Syntax error: Operator priority clash: 'and'
% Erklärung: Aus der Definition von op/3 folgt, dass die Priorität von x = harry is_a wizard mit 300
% kleiner als der von f = and mit 200 sein müsste.

% 4. ?- write_canonical(dumbledore is_a famous famous wizard or_not).
% --> or_not(is_a(dumbledore,famous(famous(wizard)))), true

% 5. ?- write_canonical(famous harry and ron are wizards).
% --> are(and(famous(harry),ron),wizards), true

% 6. ?- write_canonical(ron is_a wizards and harry likes quidditch).
% --> Syntax error: Operator expected: likes 
% Erklärung: Aus der Definition des Operator likes folgt, dass dieser die Struktur fx besitzt 
% und somit rechtsseitig kein Argument sondern einen Operator erwartet.
